unit Chess;

{
  This unit implements a non visual chess bord component.
  It handles a bord of 8x8 cells, each of which has a color and a piece property.
  A complete situation is returned by the ExportToList method. It loads a TStringList
  with 8 rows of 8 characters. The characters have the following meanings:
    * empty cell
    P white pawn
    p black pawn
    R white rook
    r black rook
    H white knight (horse)
    h black knight (horse)
    B white bishop
    b black bishop
    Q white queen
    q black queen
    K white king
    k black king
  So, counting a CR LF after each row, a complete situation is returned by 8x8 + 8x2 = 80 characters.

    Before each move, the current situation is pushed onto the history stack. This data is composed
    of a 1 byte player indication (ccWhite or ccBlack), fololowed by the 80 characters of the
    currentr situation image.


}

interface
  uses
    Windows, SysUtils, StrUtils, Messages, Classes, Controls, ExtCtrls, Graphics, ImgList,  Dialogs,
    Forms, StdCtrls, Buttons
    ;

  type
    TChessColor = (ccNone, ccWhite, ccBlack);                                     // vide, noir, blanc
    TChessPiece = (cpNone, cpPawn, cpRook, cpHorse, cpBishop, cpQueen, cpKing);   // vide, pion, fou, cheval, tout, reine, roi
    TChessRoque = (crNone, crSmall, crLarge);                                     // roque: aucun, petit roque, grand roque
    
    TChessCell = class
      private
        fColor: TChessColor;
        fPiece: TChessPiece;
      public
        property Color: TChessColor read fColor write fColor;
        property Piece: TChessPiece read fPiece write fPiece;

        constructor CreateNew;
        destructor Destroy; override;
    end;

    TChessPanel = class(TPanel)
      private
        fDark: TColor;
        fLight: TColor;
        fIconSize: integer;
        fField: array [0..7,0..7] of TChessCell;
        fImageList: TImageList;
        fImageOffset: integer;
        fStartRow: integer;
        fStartCol: integer;
        fStartPiece: char;
        fEndRow: integer;
        fEndCol: integer;
        fEndPiece: char;
        fNextMove: TChessColor;
        fMoveResult: boolean;
        fTrace: HWND;
        fWhitePawnDoubleAdvance: TPoint;
        fBlackPawnDoubleAdvance: TPoint;
        fWhiteKingMoved: boolean;
        fBlackKingMoved: boolean;
        fWhiteRookMoved: array [0..1] of boolean;
        fBlackRookMoved: array [0..1] of boolean;
        fWhiteInCheck: boolean;
        fBlackInCheck: boolean;

        function GetColor(aCol, aRow: integer): TChessColor;
        procedure SetColor(aCol, aRow: integer; aColor: TChessColor);
        function GetPiece(aCol, aRow: integer): TChessPiece;
        procedure SetPiece(aCol, aRow: integer; aPiece: TChessPiece);
        function GetLeftWhiteRookMoved: boolean;
        procedure SetLeftWhiteRookMoved(aValue: boolean);
        function GetRightWhiteRookMoved: boolean;
        procedure SetRightWhiteRookMoved(aValue: boolean);
        function GetLeftBlackRookMoved: boolean;
        procedure SetLeftBlackRookMoved(aValue: boolean);
        function GetRightBlackRookMoved: boolean;
        procedure SetRightBlackRookMoved(aValue: boolean);

      protected
        property Dark: TColor read fDark write fDark;
        property Light: TColor read fLight write fLight;
        property ImageList: TImageList read fImageList write fImageList;
        property ImageOffset: integer read fImageOffset write fImageOffset;
        property IconSize: integer read fIconSize write fIconSize;
        property StartRow: integer read fStartRow write fStartRow;
        property StartCol: integer read fStartCol write fStartCol;
        property EndRow: integer read fEndRow write fEndRow;
        property EndCol: integer read fEndCol write fEndCol;
        property StartPiece: char read fStartPiece write fStartPiece;
        property EndPiece: char read fEndPiece write fEndPiece;
        property NextMove: TChessColor read fNextMove write fNextMove;
        property MoveResult: boolean read fMoveResult write fMoveResult;
        property Trace: HWND read fTrace write fTrace;
        property Piece[aCol,aRow: integer]: TChessPiece read GetPiece write SetPiece;
        property Color[aCol,aRow: integer]: TChessColor read GetColor write SetColor;
        property WhitePawnDoubleAdvance: TPoint read fWhitePawnDoubleAdvance write fWhitePawnDoubleAdvance;
        property BlackPawnDoubleAdvance: TPoint read fBlackPawnDoubleAdvance write fBlackPawnDoubleAdvance;
        property WhiteKingMoved: boolean read fWhiteKingMoved write fWhiteKingMoved;
        property BlackKingMoved: boolean read fBlackKingMoved write fBlackKingMoved;
        property LeftWhiteRookMoved: boolean read GetLeftWhiteRookMoved write SetLeftWhiteRookMoved;
        property RightWhiteRookMoved: boolean read GetRightWhiteRookMoved write SetRightWhiteRookMoved;
        property LeftBlackRookMoved: boolean read GetLeftBlackRookMoved write SetLeftBlackRookMoved;
        property RightBlackRookMoved: boolean read GetRightBlackRookMoved write SetRightBlackRookMoved;
        property WhiteInCheck: boolean read fWhiteInCheck write fWhiteInCheck;
        property BlackInCheck: boolean read fBlackInCheck write fBlackInCheck;

        procedure Paint; override;
        procedure MyMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        procedure MyMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
        function IdentifyPiece(Row,Col: integer): char;
        procedure Move(StartCol,StartRow,EndCol,EndRow: integer);
        function IsAdverseCheck: boolean;
        function IsSelfCheck(StartCol,StartRow,EndCol,EndRow: integer): boolean;
      end;

    TChessBoard = class
      private
        fHistory: TStringList;
        fBoard: TChessPanel;
        fField: array [0..7,0..7] of TChessCell;
      public
        property Board: TChessPanel read fBoard write fBoard;

        constructor CreateNew;
        destructor Destroy; override;
        procedure Initialize;
        procedure ExportToList(aList: TStringList);
        procedure ImportFromList(aList: TStringList);
        procedure PushContext;
        procedure PopContext;
        procedure DrawBoard;
        procedure CopySituation;
    end;

function CreateChessBoardForDelphi(hDest: HWND; siz, x, y: integer; ColorDark, ColorLight: TColor): TChessBoard; stdcall; forward;
function SetChessBoardTrace(CB: TChessBoard; trace: HWND): integer; stdcall; forward;
function MoveChessBoardPiece(CB: TChessBoard; StartCol, StartRow, EndCol, EndRow: integer; Roque: TChessRoque): integer; stdcall; forward;
function PopulateChessBoard(CB: TChessBoard; aSituation: integer; StartColor: TChessColor): integer; stdcall; forward;
function DeleteChessBoard(CB: TChessBoard): TChessBoard; stdcall; forward;
function ResizeChessBoard(CB: TChessBoard; siz: integer): integer; stdcall; forward;

function DoesPieceMenaceField(CP: TChessPanel; aCol, aRow: integer; aColor: TChessColor): boolean; forward;

implementation

constructor TChessCell.CreateNew;
begin
  inherited;
  Piece := cpNone;
  Color := ccNone;
end;

Destructor TChessCell.Destroy;
begin
  inherited;
end;

procedure TChessPanel.Paint;
var
  row, col, x, y, ind: integer;
  Icon: TBitmap;
const
  ChessPieces = 'PpRrHhBbQqKk';
begin
  inherited;

  // draw dark background and border
  Canvas.Brush.Color := Dark;
  Canvas.Pen.Color := clBlack;
  Canvas.FillRect(ClientRect);
  Canvas.Rectangle(0,0,ClientWidth,ClientHeight);
  // draw horizontal lines
  x := 1;
  for row:=1 to 7 do begin
    x := x + IconSize + 1;
    Canvas.MoveTo(0,x);
    Canvas.LineTo(ClientWidth,x);
  end;
  // draw vertical liines
  x := 1;
  for col:=1 to 7 do begin
    x := x + IconSize + 1;
    Canvas.MoveTo(x,0);
    Canvas.LineTo(x,ClientHeight);
  end;
  // put light color into appropriate cells
  Canvas.Brush.Color := Light;
  y := 10;
  for row:=1 to 8 do begin
    x := 10;
    for col:=1 to 8 do begin
      if (col and 1)=(row and 1) then begin
        Canvas.FloodFill(x,y,Dark,fsSurface);
      end;
      x := x + IconSize + 1;
    end;
    y := y + IconSize + 1;
  end;
  // place the pieces onto the chessboard
    for row:=0 to 7 do begin
      for col:=0 to 7 do begin
        if fField[row,col].Piece<>cpNone then begin
          x := 1+(7-col)*(IconSize+1) + 1;
          y := 1+row*(IconSize+1) + 1;
          Icon := TBitmap.Create;
          Icon.Width := IconSize;
          Icon.Height := IconSize;
          ind := 2*(ord(fField[row,col].Piece)-1) + ord(fField[row,col].Color)-1;
          ImageList.GetBitmap(ind+ImageOffset,Icon);
          Icon.Canvas.Brush.Color := Canvas.Pixels[y,x];
          Icon.Canvas.FloodFill(0,0,Icon.Canvas.Pixels[0,0],fsSurface);
          Canvas.StretchDraw(rect(y,x,y+IconSize-1,x+IconSize-1),Icon);
//          Canvas.Draw(y,x,Icon);
          Icon.Free;
        end;
      end;
    end;
end;

function TChessPanel.GetColor(aCol, aRow: integer): TChessColor;
begin
  result := fField[aCol,aRow].Color;
end;

procedure TChessPanel.SetColor(aCol, aRow: integer; aColor: TChessColor);
begin
  fField[aCol,aRow].Color := aColor;
end;

function TChessPanel.GetPiece(aCol, aRow: integer): TChessPiece;
begin
  result := fField[aCol,aRow].Piece;
end;

procedure TChessPanel.SetPiece(aCol, aRow: integer; aPiece: TChessPiece);
begin
  fField[aCol,aRow].Piece := aPiece;
end;

function TChessPanel.GetLeftWhiteRookMoved: boolean;
begin
  result := fWhiteRookMoved[0];
end;

procedure TChessPanel.SetLeftWhiteRookMoved(aValue: boolean);
begin
  fWhiteRookMoved[0] := aValue;
end;

function TChessPanel.GetRightWhiteRookMoved: boolean;
begin
  result := fWhiteRookMoved[1];
end;

procedure TChessPanel.SetRightWhiteRookMoved(aValue: boolean);
begin
  fWhiteRookMoved[1] := aValue;
end;

function TChessPanel.GetLeftBlackRookMoved: boolean;
begin
  result := fBlackRookMoved[0];
end;

procedure TChessPanel.SetLeftBlackRookMoved(aValue: boolean);
begin
  fBlackRookMoved[0] := aValue;
end;

function TChessPanel.GetRightBlackRookMoved: boolean;
begin
  result := fBlackRookMoved[1];
end;

procedure TChessPanel.SetRightBlackRookMoved(aValue: boolean);
begin
  fBlackRookMoved[1] := aValue;
end;


// move a piece unconditionnally
procedure TChessPanel.Move(StartCol,StartRow,EndCol,EndRow: integer);
begin
  fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
  fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
  fField[StartCol,StartRow].Piece := cpNone;
  fField[StartCol,StartRow].Color := ccNone;
end;

// test if adverse king is in check situation
function TChessPanel.IsAdverseCheck: boolean;
var
  TestColor: TChessColor;
  KingCol, KingRow, col, row: integer;
  found: boolean;

  function GetKing: boolean;
  var
    row, col: integer;
  begin
    result := false;
    for row:=0 to 7 do begin
      for col:=0 to 7 do begin
        if Color[col,row]=TestColor then begin
          if Piece[col,row]=cpKing then begin
            KingCol := col;
            KingRow := row;
            result := true;
            exit;
          end;
        end;
      end;
    end;
  end;

begin
  result := false;
  if self.NextMove=ccWhite then TestColor := ccBlack
                         else TestColor := ccWhite;
  if not GetKing then exit;
  result := DoesPieceMenaceField(self,KingCol,KingRow,NextMove);
  if result then begin
    if Color[KingCol,KingRow]=ccWhite then WhiteInCheck := true
                                      else BlackInCheck := true;
  end else begin
    WhiteInCheck := false;
    BlackInCheck := false;
  end;
end;


// test if own king is in check situation
function TChessPanel.IsSelfCheck(StartCol,StartRow,EndCol,EndRow: integer): boolean;
var
  TestColor: TChessColor;
  KingCol, KingRow, col, row: integer;
  found: boolean;
  StartCell, EndCell: TChessCell;

  function GetKing: boolean;
  var
    row, col: integer;
  begin
    result := false;
    for row:=0 to 7 do begin
      for col:=0 to 7 do begin
        if Color[col,row]=TestColor then begin
          if Piece[col,row]=cpKing then begin
            KingCol := col;
            KingRow := row;
            result := true;
            exit;
          end;
        end;
      end;
    end;
  end;

begin
  result := false;
  TestColor := NextMove;
  if not GetKing then exit;
  if NextMove=ccWhite then TestColor := ccBlack
                         else TestColor := ccWhite;

  StartCell := TChessCell.CreateNew;
  EndCell := TChessCell.CreateNew;

  StartCell.Color := Color[StartCol,StartRow];
  StartCell.Piece := Piece[StartCol,StartRow];
  EndCell.Color := Color[EndCol,EndRow];
  EndCell.Piece := Piece[EndCol,EndRow];

  Color[StartCol,StartRow] := ccNone;
  Piece[StartCol,StartRow] := cpNone;
  Color[EndCol,EndRow] := StartCell.Color;
  Piece[EndCol,EndRow] := StartCell.Piece;

  result := DoesPieceMenaceField(self,KingCol,KingRow,TestColor);

  Color[StartCol,StartRow] := StartCell.Color;
  Piece[StartCol,StartRow] := StartCell.Piece;
  Color[EndCol,EndRow] := EndCell.Color;
  Piece[EndCol,EndRow] := EndCell.Piece;
  StartCell.Free;
  EndCell.Free;

  if result then begin
    if Color[KingCol,KingRow]=ccWhite then WhiteInCheck := true
                                      else BlackInCheck := true;
  end else begin
    WhiteInCheck := false;
    BlackInCheck := false;
  end;
end;


// return an identifying character for the piece withina cell
function TChessPanel.IdentifyPiece(Row,Col: integer): char;
begin
  case fField[Col,Row].Color of
    ccNone: result := '*';               // empty cell
    ccWhite: begin
      case fField[Col,Row].Piece of      // white pieces use upper case letters
        cpNone: result := '*';
        cpPawn: result := 'P';
        cpRook: result := 'R';
        cpHorse: result := 'H';
        cpBishop: result := 'B';
        cpQueen: result := 'Q';
        cpKing: result := 'K';
      end;
    end;
    ccBlack: begin
      case fField[Col,Row].Piece of      // black pieces use lower case letters
        cpNone: result := '*';
        cpPawn: result := 'p';
        cpRook: result := 'r';
        cpHorse: result := 'h';
        cpBishop: result := 'b';
        cpQueen: result := 'q';
        cpKing: result := 'k';
      end;
    end;
  end;
end;

// mouse down event on the chessboard: identifies the piece to move
procedure TChessPanel.MyMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  s: string;
  r, c: integer;
begin
  MoveResult := false;
  if Button = mbLeft then begin

    if (x>=(width-4)) or (x<=4) or (y<=4) or (y>=(Height-4)) then begin
      inherited;
      exit;
    end;
    r := 9 - trunc((y+IconSize)/(IconSize+1)) - 1;
    c := trunc((x+IconSize)/(IconSize+1)) - 1;
    if (r<0) or (r>7) or (c<0) or (c>7) then begin
      inherited;
      exit;
    end;
    StartCol := c;
    StartRow := r;

    if (x>=(width-2)) or (x<2) then begin
      inherited;
      exit;
    end;

    StartPiece := IdentifyPiece(StartRow,StartCol);

s := 'down: '+inttostr(StartCol)+','+inttostr(StartRow)+' = '+StartPiece;
if Trace<>0 then sendmessage(Trace,WM_SETTEXT,0,integer(@s[1]));

  end else inherited;
end;

// if a pawn reaches the adverse ground line, it will be replaced by an officer choosen from a small list
function SelectOfficer(pnt: TPoint): TChessPiece;
var
  dlg: TForm;
  RG: TRadioGroup;
  res: integer;
  RBRook, RBHorse, RBBishop, RBQueen: TRadioButton;
begin
  result := cpQueen;

  dlg := TForm.Create(nil);
  dlg.Caption := 'Choix de l''officier';
  dlg.Width := 150;
  dlg.Height := 140;
  dlg.Left := pnt.X;
  dlg.Top := pnt.Y;

  RG := TRadioGroup.Create(dlg);
  RG.Caption := 'Officiers possibles';
  RG.Top := 0;
  RG.Left := 0;
  RG.Width := dlg.ClientWidth;
  RG.Height := dlg.ClientHeight;
  RG.ParentWindow := dlg.Handle;

  RBRook := TRadioButton.Create(RG);
  RBRook.Caption := 'Tour';
  RBRook.Top := 20;
  RBRook.Left := 20;
  RG.InsertControl(RBRook);

  RBHorse := TRadioButton.Create(RG);
  RBHorse.Caption := 'Cavalier';
  RBHorse.Top := RBRook.Top+20;
  RBHorse.Left := 20;
  RG.InsertControl(RBHorse);

  RBBishop := TRadioButton.Create(RG);
  RBBishop.Caption := 'Fou';
  RBBishop.Top := RBHorse.Top+20;
  RBBishop.Left := 20;
  RG.InsertControl(RBBishop);

  RBQueen := TRadioButton.Create(RG);
  RBQueen.Caption := 'Reine';
  RBQueen.Top := RBBishop.Top+20;
  RBQueen.Left := 20;
  RBQueen.Checked := true;
  RG.InsertControl(RBQueen);

  res := dlg.ShowModal;

  if RBRook.Checked then result := cpRook;
  if RBHorse.Checked then result := cpHorse;
  if RBBishop.Checked then result := cpBishop;
  if RBQueen.Checked then result := cpQueen;

  dlg.Free;
end;

// check if any piece of a given color menaces a given field
function DoesPieceMenaceField(CP: TChessPanel; aCol, aRow: integer; aColor: TChessColor): boolean;
var
  col, row, c, r, i: integer;
  aPiece: TChessPiece;
  flag: boolean;

  // internal function: check if a rook menaces the given field
  function CheckRook: boolean;
  var
    r, c: integer;
  begin
    result := true;
                    flag := true;
                    if aCol=col then begin
                      if row>aRow+1 then begin
                        for r:=aRow+1 to row-1 do if CP.Piece[aCol,r]<>cpNone then flag := false;
                        if flag then exit;
                      end;
                      if row<aRow-1 then begin
                        for r:=row+1 to aRow-1 do if CP.Piece[aCol,r]<>cpNone then flag := false;
                        if flag then exit;
                      end;
                    end;
                    if aRow=row then begin
                      if col>aCol+1 then begin
                        for c:=aCol+1 to col-1 do if CP.Piece[c,aRow]<>cpNone then flag := false;
                        if flag then exit;
                      end;
                      if col<aCol-1 then begin
                        for c:=col+1 to aCol-1 do if CP.Piece[c,aRow]<>cpNone then flag := false;
                        if flag then exit;
                      end;
                    end;
    result := false;
  end;

  // internal function: check if a bishop menaces the given field
  function CheckBishop: boolean;
  var
    i: integer;
  begin
    result := true;
                      flag := true;
                      if abs(aCol-col)=abs(aRow-row) then begin
                        if col<aCol-1 then begin
                          if row<aRow then begin
                            for i:=1 to aCol-col-1 do if CP.Piece[col+i,row+i]<>cpNone then flag := false;
                            if flag then exit;
                          end;
                          if row>aRow then begin
                            for i:=1 to aCol-col-1 do if CP.Piece[col+i,row-i]<>cpNone then flag := false;
                            if flag then exit;
                          end;
                        end else begin
                          if row<aRow then begin
                            for i:=1 to col-aCol-1 do if CP.Piece[col-i,row+i]<>cpNone then flag := false;
                            if flag then exit;
                          end;
                          if row>aRow then begin
                            for i:=1 to col-aCol-1 do if CP.Piece[col-i,row-i]<>cpNone then flag := false;
                            if flag then exit;
                          end;
                        end;
                      end;
    result := false;
  end;

begin
  result := true;
  for row:=0 to 7 do begin
    for col:=0 to 7 do begin
      if CP.Color[col,row]=aColor then begin
        aPiece := CP.Piece[col,row];
        case aPiece of
          cpPawn: begin
                    case aColor of
                      ccWhite: begin
                                 if (aCol=col-1) and (aRow=row+1) then exit;
                                 if (aCol=col+1) and (aRow=row+1) then exit;
                               end;
                      ccBlack: begin
                                 if (aCol=col-1) and (aRow=row-1) then exit;
                                 if (aCol=col+1) and (aRow=row-1) then exit;
                               end;
                    end;
                  end;
          cpRook: begin
                    if CheckRook then exit;
                  end;
          cpHorse: begin
                     if (aCol=col-1) and (aRow=row-2) then exit;
                     if (aCol=col+1) and (aRow=row-2) then exit;
                     if (aCol=col+2) and (aRow=row-1) then exit;
                     if (aCol=col+2) and (aRow=row+1) then exit;
                     if (aCol=col+1) and (aRow=row+2) then exit;
                     if (aCol=col-1) and (aRow=row+2) then exit;
                     if (aCol=col-2) and (aRow=row+1) then exit;
                     if (aCol=col-2) and (aRow=row-1) then exit;
                   end;
          cpBishop: begin
                      if CheckBishop then exit;
                    end;
          cpQueen: begin
                      if CheckRook then exit;
                      if CheckBishop then exit;
                   end;
          cpKing: begin
                    if (col=aCol) then if (row=aRow-1) or (row=aRow+1) then exit;
                    if (col=aCol-1) then if (row=aRow-1) or (row=aRow+1) then exit;
                    if (col=aCol+1) then if (row=aRow-1) or (row=aRow+1) then exit;
                  end;
        end;
      end;
    end;
  end;
  result := false;
end;

// check if a rook is legal for the active color
function IsRoqueLegal(CP: TChessPanel; Roque: TChessRoque): boolean; stdcall;
{
  Les conditions suivantes sont ncessaires pour pouvoir roquer :
    1. aucune pice ne se trouve entre le roi et la tour concerne ;
    2. le roi et la tour concerne n'ont encore jamais jou ;
    3. le roi n'est pas en chec ;
    4. la case traverse par le roi n'est contrle par aucune pice adverse.
  The followong conditions must be valid before a rook can happen:
    1. no other piece must be present between the king and the concerned rook
    2. the king and the concerned rook must never have moved
    3. the king must not actually be menaced
    4. the case traversed by the king must not be menaced by any adverse piece
}
var
  col: integer;
begin
  result := false;                                                                 // technical error
  try
    case Roque of
      crSmall: begin         // petit roque du joueur dont c'est le tour
                 if CP.NextMove=ccWhite then begin
                   //  1. aucune pice ne se trouve entre le roi et la tour concerne ;
                   if (CP.Piece[4,0]<>cpKing) and (CP.Color[4,0]<>ccWhite) then exit;
                   if (CP.Piece[5,0]<>cpNone) then exit;
                   if (CP.Piece[6,0]<>cpNone) then exit;
                   if (CP.Piece[7,0]<>cpRook) and (CP.Color[7,0]<>ccWhite) then exit;
                   //  2. le roi et la tour concerne n'ont encore jamais jou ;
                   if CP.WhiteKingMoved then exit;
                   if CP.RightWhiteRookMoved then exit;
                   //  3. le roi n'est pas en chec ;
                   //  4. la case traverse par le roi n'est contrle par aucune pice adverse.
                   for col:=4 to 6 do if DoesPieceMenaceField(CP,col,0,ccBlack) then exit;
                 end else begin
                   //  1. aucune pice ne se trouve entre le roi et la tour concerne ;
                   if (CP.Piece[4,7]<>cpKing) and (CP.Color[4,7]<>ccBlack) then exit;
                   if (CP.Piece[5,7]<>cpNone) then exit;
                   if (CP.Piece[6,7]<>cpNone) then exit;
                   if (CP.Piece[7,7]<>cpRook) and (CP.Color[7,7]<>ccBlack) then exit;
                   //  2. le roi et la tour concerne n'ont encore jamais jou ;
                   if CP.BlackKingMoved then exit;
                   if CP.RightBlackRookMoved then exit;
                   //  3. le roi n'est pas en chec ;
                   //  4. la case traverse par le roi n'est contrle par aucune pice adverse.
                   for col:=4 to 6 do if DoesPieceMenaceField(CP,col,7,ccWhite) then exit;
                 end;
               end;
      crLarge: begin         // grand roque du joueur dont c'est le tour
                 if CP.NextMove=ccWhite then begin
                   //  1. aucune pice ne se trouve entre le roi et la tour concerne ;
                   if (CP.Piece[4,0]<>cpKing) and (CP.Color[4,0]<>ccWhite) then exit;
                   if (CP.Piece[3,0]<>cpNone) then exit;
                   if (CP.Piece[2,0]<>cpNone) then exit;
                   if (CP.Piece[1,0]<>cpNone) then exit;
                   if (CP.Piece[0,0]<>cpRook) and (CP.Color[0,0]<>ccWhite) then exit;
                   //  2. le roi et la tour concerne n'ont encore jamais jou ;
                   if CP.WhiteKingMoved then exit;
                   if CP.LeftWhiteRookMoved then exit;
                   //  3. le roi n'est pas en chec ;
                   //  4. la case traverse par le roi n'est contrle par aucune pice adverse.
                   for col:=1 to 4 do if DoesPieceMenaceField(CP,col,0,ccBlack) then exit;
                 end else begin
                   //  1. aucune pice ne se trouve entre le roi et la tour concerne ;
                   if (CP.Piece[4,7]<>cpKing) and (CP.Color[4,7]<>ccBlack) then exit;
                   if (CP.Piece[3,7]<>cpNone) then exit;
                   if (CP.Piece[2,7]<>cpNone) then exit;
                   if (CP.Piece[1,7]<>cpNone) then exit;
                   if (CP.Piece[0,7]<>cpRook) and (CP.Color[0,7]<>ccBlack) then exit;
                   //  2. le roi et la tour concerne n'ont encore jamais jou ;
                   if CP.BlackKingMoved then exit;
                   if CP.LeftBlackRookMoved then exit;
                   //  3. le roi n'est pas en chec ;
                   //  4. la case traverse par le roi n'est contrle par aucune pice adverse.
                   for col:=1 to 4 do if DoesPieceMenaceField(CP,col,7,ccWhite) then exit;
                 end;
               end;
    end;
    result := true;
  except
  end;
end;

// mouse up event: determines the field where the targetted piece will be dropped
// here, all rules are checked before executing the move
procedure TChessPanel.MyMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  s: string;
  ind, i, r, c: integer;
  bad, victory, possible, EnPassant, DoRoque: boolean;
const
  ChessPieces = 'PpRrHhBbQqKk';
begin
  if Button = mbLeft then begin
  MoveResult := false;

  victory := false;
  if (x>=(width-4)) or (x<=4) or (y<=4) or (y>=(Height-4)) then begin
    inherited;
    exit;
  end;
  r := 9 - trunc((y+IconSize)/(IconSize+1)) - 1;
  c := trunc((x+IconSize)/(IconSize+1)) - 1;
  if (r<0) or (r>7) or (c<0) or (c>7) then begin
    inherited;
    exit;
  end;
  EndCol := c;
  EndRow := r;

  EndPiece := IdentifyPiece(EndRow,EndCol);

s := 'up: '+inttostr(EndCol)+','+inttostr(EndRow)+' = '+EndPiece;
if Trace<>0 then sendmessage(Trace,WM_SETTEXT,0,integer(@s[1]));

  if NextMove<>fField[StartCol,StartRow].Color then exit;                      // not expected player color ?
  if (StartRow<>EndRow) or (StartCol<>EndCol) then begin                        // a move is intended ?

    ind := pos(StartPiece,ChessPieces) - 1;
    case ind of                            // dispatch en fonction de la pice vise
      0: begin   // white pawn
           // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           EnPassant := false;

           bad := false;
           if (EndRow<=StartRow) or (EndRow>StartRow+2) then bad := true;
           if not bad then  if (StartRow>1) and (EndRow>StartRow+1) then bad := true;
           if not bad then if StartCol<>EndCol then begin
             if (StartCol<>EndCol-1) and (StartCol<>EndCol+1) then bad := true;
             if not bad then if EndRow=5 then begin
               if (EndCol=BlackPawnDoubleAdvance.X) and (fField[EndCol,EndRow].Piece=cpNone) then EnPassant := true;
             end;
             if (not bad) and (not EnPassant) then if fField[EndCol,EndRow].Piece=cpNone then bad := true;
             if not bad then if (fField[EndCol,EndRow].Piece<>cpNone) and (fField[EndCol,EndRow].Color=ccWhite) then bad := true;
           end else begin
             if not bad then if (EndRow=StartRow+2) and (fField[EndCol,EndRow-1].Piece<>cpNone) then bad := true;
             if not bad then if fField[EndCol,EndRow].Piece<>cpNone then bad := true;
           end;
           if not bad then if WhiteInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le pion blanc
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             if StartRow=EndRow-2 then WhitePawnDoubleAdvance := Point(EndCol,EndRow);

             s := 'Dplacer Pion blanc de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             if EnPassant then begin
               fField[EndCol,EndRow-1].Piece := cpNone;
               fField[EndCol,EndRow-1].Color := ccNone;
             end;
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccBlack);
             if victory then s := s + '  *** VICTOIRE des BLANCS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             if EndRow=7 then fField[EndCol,EndRow].Piece := SelectOfficer(ClientToScreen(Point(x,y)));     // white pawn gains a white officer !
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccBlack;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      1: begin   // black pawn
           // vrifier l'autorisation de dplacement
           BlackPawnDoubleAdvance := Point(-1,-1);
           EnPassant := false;

           bad := false;
           if (EndRow>=StartRow) or (EndRow<StartRow-2) then bad := true;
           if not bad then  if (StartRow<6) and (EndRow<StartRow-1) then bad := true;
           if not bad then if StartCol<>EndCol then begin
             if (StartCol<>EndCol-1) and (StartCol<>EndCol+1) then bad := true;
             if not bad then if EndRow=2 then begin
               if (EndCol=WhitePawnDoubleAdvance.X) and (fField[EndCol,EndRow].Piece=cpNone) then EnPassant := true;
             end;
             if (not bad) and (not EnPassant) then if fField[EndCol,EndRow].Piece=cpNone then bad := true;
             if not bad then if (fField[EndCol,EndRow].Piece<>cpNone) and (fField[EndCol,EndRow].Color=ccBlack) then bad := true;
           end else begin
             if not bad then if (EndRow=StartRow-2) and (fField[EndCol,EndRow+1].Piece<>cpNone) then bad := true;
             if not bad then if fField[EndCol,EndRow].Piece<>cpNone then bad := true;
           end;
           if not bad then if BlackInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le pion noir
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             if StartRow=EndRow+2 then BlackPawnDoubleAdvance := Point(EndCol,EndRow);

             s := 'Dplacer Pion noir de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             if EnPassant then begin
               fField[EndCol,EndRow+1].Piece := cpNone;
               fField[EndCol,EndRow+1].Color := ccNone;
             end;
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccWhite);
             if victory then s := s + '  *** VICTOIRE des NOIRS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             if EndRow=0 then fField[EndCol,EndRow].Piece := SelectOfficer(ClientToScreen(Point(x,y)));     // black pawn gains a black officer !
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccWhite;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      2: begin      // white rook
           // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if (StartRow<>EndRow) and (StartCol<>EndCol) then bad := true;
           if not bad then if StartRow=EndRow then begin
             if StartCol<EndCol-1 then begin
               for i:=StartCol+1 to EndCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
             end else begin
               if StartCol>EndCol+1 then begin
                 for i:=EndCol+1 to StartCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
               end;
             end;
           end else begin
             if StartRow<EndRow-1 then begin
               for i:=StartRow+1 to EndRow-1 do if fField[StartCol,i].Piece<>cpNone then bad := true;
             end else begin
               for i:=EndRow+1 to StartRow-1 do  if fField[StartCol,i].Piece<>cpNone then bad := true;
             end;
           end;
           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccWhite then bad := true;
           end;
           if not bad then if WhiteInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le tour blanche
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Tour blanche de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             if StartCol=0 then self.LeftWhiteRookMoved := true;
             if StartCol=7 then self.RightWhiteRookMoved := true;
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccBlack);
             if victory then s := s + '  *** VICTOIRE des BLANCS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccBlack;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      3: begin    // black rook
           // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if (StartRow<>EndRow) and (StartCol<>EndCol) then bad := true;
           if not bad then if StartRow=EndRow then begin
             if StartCol<EndCol-1 then begin
               for i:=StartCol+1 to EndCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
             end else begin
               if StartCol>EndCol+1 then begin
                 for i:=EndCol+1 to StartCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
               end;
             end;
           end else begin
             if StartRow<EndRow-1 then begin
               for i:=StartRow+1 to EndRow-1 do if fField[StartCol,i].Piece<>cpNone then bad := true;
             end else begin
               for i:=EndRow+1 to StartRow-1 do  if fField[StartCol,i].Piece<>cpNone then bad := true;
             end;
           end;
           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccBlack then bad := true;
           end;
           if not bad then if BlackInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le tour noire
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Tour noire de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             if StartCol=0 then self.LeftBlackRookMoved := true;
             if StartCol=7 then self.RightBlackRookMoved := true;
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccWhite);
             if victory then s := s + '  *** VICTOIRE des NOIRS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccWhite;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));         end;
      4: begin    // white knight
           // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           possible := false;
           if StartRow=EndRow-1 then begin
             if (StartCol<>EndCol-2) and (StartCol<>EndCol+2) then bad := true;
             possible := true;
           end;
           if not bad then if StartRow=EndRow+1 then begin
             if (StartCol<>EndCol-2) and (StartCol<>EndCol+2) then bad := true;
             possible := true;
           end;
           if not bad then if StartCol=EndCol-1 then begin
             if (StartRow<>EndRow-2) and (StartRow<>EndRow+2) then bad := true;
             possible := true;
           end;
           if not bad then if StartCol=EndCol+1 then begin
             if (StartRow<>EndRow-2) and (StartRow<>EndRow+2) then bad := true;
             possible := true;
           end;
           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccWhite then bad := true;
           end;
           if not possible then bad := true;
           if not bad then if WhiteInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;
           
           // dplacer le cavalier blanc
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Cavalier blanc de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccBlack);
             if victory then s := s + '  *** VICTOIRE des BLANCS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccBlack;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      5: begin    // black knight
   // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           possible := false;
           if StartRow=EndRow-1 then begin
             if (StartCol<>EndCol-2) and (StartCol<>EndCol+2) then bad := true;
             possible := true;
           end;
           if not bad then if StartRow=EndRow+1 then begin
             if (StartCol<>EndCol-2) and (StartCol<>EndCol+2) then bad := true;
             possible := true;
           end;
           if not bad then if StartCol=EndCol-1 then begin
             if (StartRow<>EndRow-2) and (StartRow<>EndRow+2) then bad := true;
             possible := true;
           end;
           if not bad then if StartCol=EndCol+1 then begin
             if (StartRow<>EndRow-2) and (StartRow<>EndRow+2) then bad := true;
             possible := true;
           end;
           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccBlack then bad := true;
           end;
           if not possible then bad := true;
           if not bad then if BlackInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le cavalier noir
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Cavalier noir de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccWhite);
             if victory then s := s + '  *** VICTOIRE des NOIRS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccWhite;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      6: begin       // white bishop
           // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if abs(StartRow-EndRow)<>abs(StartCol-EndCol) then bad := true;
           if not bad then if (EndRow-StartRow)>1 then begin
             if EndCol>StartCol then begin
               for i:=1 to (EndRow-Startrow-1) do begin
                 if not bad then if fField[Startcol+i,StartRow+i].Piece<>cpNone then bad := true;
               end;
             end else begin
               for i:=1 to (EndRow-Startrow-1) do begin
                 if not bad then if fField[Endcol+i,EndRow-i].Piece<>cpNone then bad := true;
               end;
             end;
           end;
           if not bad then if (StartRow-EndRow)>1 then begin
             if EndCol>StartCol then begin
               for i:=1 to (StartRow-Endrow-1) do begin
                 if not bad then if fField[Startcol+i,StartRow-i].Piece<>cpNone then bad := true;
               end;
             end else begin
               for i:=1 to (StartRow-Endrow-1) do begin
                 if not bad then if fField[Endcol+i,EndRow+i].Piece<>cpNone then bad := true;
               end;
             end;
           end;

           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccWhite then bad := true;
           end;
           if not bad then if WhiteInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le fou blanc
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Fou blanc de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccBlack);
             if victory then s := s + '  *** VICTOIRE des BLANCS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccBlack;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      7: begin     // black bishop
           // vrifier l'autorisation de dplacement
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if abs(StartRow-EndRow)<>abs(StartCol-EndCol) then bad := true;
           if not bad then if (EndRow-StartRow)>1 then begin
             if EndCol>StartCol then begin
               for i:=1 to (EndRow-Startrow-1) do begin
                 if not bad then if fField[Startcol+i,StartRow+i].Piece<>cpNone then bad := true;
               end;
             end else begin
               for i:=1 to (EndRow-Startrow-1) do begin
                 if not bad then if fField[Endcol+i,EndRow-i].Piece<>cpNone then bad := true;
               end;
             end;
           end;
           if not bad then if (StartRow-EndRow)>1 then begin
             if EndCol>StartCol then begin
               for i:=1 to (StartRow-Endrow-1) do begin
                 if not bad then if fField[Startcol+i,StartRow-i].Piece<>cpNone then bad := true;
               end;
             end else begin
               for i:=1 to (StartRow-Endrow-1) do begin
                 if not bad then if fField[Endcol+i,EndRow+i].Piece<>cpNone then bad := true;
               end;
             end;
           end;

           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccBlack then bad := true;
           end;
           if not bad then if BlackInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le fou noir
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Fou noir de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccWhite);
             if victory then s := s + '  *** VICTOIRE des NOIRS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccWhite;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      8: begin     // white queen
           // vrifier l'autorisation de dplacement
           // contrle style "tour"
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if (StartRow<>EndRow) and (StartCol<>EndCol) then bad := true;
           if not bad then if StartRow=EndRow then begin
             if StartCol<EndCol-1 then begin
               for i:=StartCol+1 to EndCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
             end else begin
               if StartCol>EndCol+1 then begin
                 for i:=EndCol+1 to StartCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
               end;
             end;
           end else begin
             if StartRow<EndRow-1 then begin
               for i:=StartRow+1 to EndRow-1 do if fField[StartCol,i].Piece<>cpNone then bad := true;
             end else begin
               for i:=EndRow+1 to StartRow-1 do  if fField[StartCol,i].Piece<>cpNone then bad := true;
             end;
           end;
           // contrle style "fou"
           if bad then begin
             bad := false;
             if not bad then if abs(StartRow-EndRow)<>abs(StartCol-EndCol) then bad := true;
             if not bad then if (EndRow-StartRow)>1 then begin
               if EndCol>StartCol then begin
                 for i:=1 to (EndRow-Startrow-1) do begin
                   if not bad then if fField[Startcol+i,StartRow+i].Piece<>cpNone then bad := true;
                 end;
               end else begin
                 for i:=1 to (EndRow-Startrow-1) do begin
                   if not bad then if fField[Endcol+i,EndRow-i].Piece<>cpNone then bad := true;
                 end;
               end;
             end;
             if not bad then if (StartRow-EndRow)>1 then begin
               if EndCol>StartCol then begin
                 for i:=1 to (StartRow-Endrow-1) do begin
                   if not bad then if fField[Startcol+i,StartRow-i].Piece<>cpNone then bad := true;
                 end;
               end else begin
                 for i:=1 to (StartRow-Endrow-1) do begin
                   if not bad then if fField[Endcol+i,EndRow+i].Piece<>cpNone then bad := true;
                 end;
               end;
             end;
           end;

           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccWhite then bad := true;
           end;
           if not bad then if WhiteInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer la reine blanche
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Reine blanche de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccBlack);
             if victory then s := s + '  *** VICTOIRE des BLANCS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccBlack;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      9: begin     // black queen
           // vrifier l'autorisation de dplacement
           // contrle style "tour"
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if (StartRow<>EndRow) and (StartCol<>EndCol) then bad := true;
           if not bad then if StartRow=EndRow then begin
             if StartCol<EndCol-1 then begin
               for i:=StartCol+1 to EndCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
             end else begin
               if StartCol>EndCol+1 then begin
                 for i:=EndCol+1 to StartCol-1 do if fField[i,StartRow].Piece<>cpNone then bad := true;
               end;
             end;
           end else begin
             if StartRow<EndRow-1 then begin
               for i:=StartRow+1 to EndRow-1 do if fField[StartCol,i].Piece<>cpNone then bad := true;
             end else begin
               for i:=EndRow+1 to StartRow-1 do  if fField[StartCol,i].Piece<>cpNone then bad := true;
             end;
           end;
           // contrle style "fou"
           if bad then begin
             bad := false;
             if not bad then if abs(StartRow-EndRow)<>abs(StartCol-EndCol) then bad := true;
             if not bad then if (EndRow-StartRow)>1 then begin
               if EndCol>StartCol then begin
                 for i:=1 to (EndRow-Startrow-1) do begin
                   if not bad then if fField[Startcol+i,StartRow+i].Piece<>cpNone then bad := true;
                 end;
               end else begin
                 for i:=1 to (EndRow-Startrow-1) do begin
                   if not bad then if fField[Endcol+i,EndRow-i].Piece<>cpNone then bad := true;
                 end;
               end;
             end;
             if not bad then if (StartRow-EndRow)>1 then begin
               if EndCol>StartCol then begin
                 for i:=1 to (StartRow-Endrow-1) do begin
                   if not bad then if fField[Startcol+i,StartRow-i].Piece<>cpNone then bad := true;
                 end;
               end else begin
                 for i:=1 to (StartRow-Endrow-1) do begin
                   if not bad then if fField[Endcol+i,EndRow+i].Piece<>cpNone then bad := true;
                 end;
               end;
             end;
           end;

           if fField[EndCol,EndRow].Piece<>cpNone then begin
             if fField[EndCol,EndRow].Color=ccBlack then bad := true;
           end;
           if not bad then if BlackInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer la reine noire
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             s := 'Dplacer Reine noire de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccWhite);
             if victory then s := s + '  *** VICTOIRE des NOIRS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccWhite;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      10: begin    // white king
           // vrifier l'autorisation de dplacement
           DoRoque := false;
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if abs(StartCol-EndCol)=2 then begin
             if EndCol=6 then if IsRoqueLegal(self,crSmall) then DoRoque := true;
             if EndCol=2 then if IsRoqueLegal(self,crLarge) then DoRoque := true;
             if not DoRoque then bad := true;
           end;
           if not DoRoque then begin
             if not bad then if (abs(StartRow-EndRow)>1) or (abs(StartCol-EndCol)>1) then bad := true;

             if not bad then if fField[EndCol,EndRow].Piece<>cpNone then begin
               if fField[EndCol,EndRow].Color=ccWhite then bad := true;
             end;
           end;
           if not bad then if WhiteInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le roi blanc
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             if DoRoque then begin
               WhiteKingMoved := true;
               if EndCol=6 then begin
                 s := 'Petit roque blanc';
                 Move(4,0,6,0);
                 Move(7,0,5,0);
                 MoveResult := true;
                 NextMove := ccBlack;
                 Paint;
               end else begin
                 s := 'Grand roque blanc';
                 Move(4,0,2,0);
                 Move(0,0,3,0);
                 MoveResult := true;
                 NextMove := ccBlack;
                 Paint;
               end;
               // log
               if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
               exit;
             end;
             s := 'Dplacer Roi blanc de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             WhiteKingMoved := true;
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccBlack);
             if victory then s := s + '  *** VICTOIRE des BLANCS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccBlack;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
      11: begin   // black king
           // vrifier l'autorisation de dplacement
           DoRoque := false;
           WhitePawnDoubleAdvance := Point(-1,-1);
           BlackPawnDoubleAdvance := Point(-1,-1);
           bad := false;
           if abs(StartCol-EndCol)=2 then begin
             if EndCol=6 then if IsRoqueLegal(self,crSmall) then DoRoque := true;
             if EndCol=2 then if IsRoqueLegal(self,crLarge) then DoRoque := true;
             if not DoRoque then bad := true;
           end;
           if not DoRoque then begin
             if not bad then if (abs(StartRow-EndRow)>1) or (abs(StartCol-EndCol)>1) then bad := true;

             if not bad then if fField[EndCol,EndRow].Piece<>cpNone then begin
               if fField[EndCol,EndRow].Color=ccBlack then bad := true;
             end;
           end;
           if not bad then if BlackInCheck then if IsSelfCheck(StartCol,StartRow,EndCol,EndRow) then bad := true;

           // dplacer le roi noir
           if bad then begin
             s := 'Mouvement impossible';
           end else begin
             if DoRoque then begin
               BlackKingMoved := true;
               if EndCol=6 then begin
                 s := 'Petit roque noir';
                 Move(4,7,6,7);
                 Move(7,7,5,7);
                 MoveResult := true;
                 NextMove := ccBlack;
                 Paint;
               end else begin
                 s := 'Grand roque noir';
                 Move(4,7,2,7);
                 Move(0,7,3,7);
                 MoveResult := true;
                 NextMove := ccBlack;
                 Paint;
               end;
               // log
               if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
               exit;
             end;
             s := 'Dplacer Roi noir de '+chr(64+StartCol)+chr(48+StartRow)+' en '+chr(64+EndCol)+chr(48+EndRow);
             BlackKingMoved := true;
             victory := (fField[EndCol,EndRow].Piece=cpKing) and (fField[EndCol,EndRow].Color=ccWhite);
             if victory then s := s + '  *** VICTOIRE des NOIRS ***';
             fField[EndCol,EndRow].Piece := fField[StartCol,StartRow].Piece;
             fField[EndCol,EndRow].Color := fField[StartCol,StartRow].Color;
             fField[StartCol,StartRow].Piece := cpNone;
             fField[StartCol,StartRow].Color := ccNone;
             if self.IsAdverseCheck then begin
               if NextMove=ccBlack then s := s + '   Blanc est en ECHEC'
                                   else s := s + '   Noir est en ECHEC';
             end;
             NextMove := ccWhite;
             if victory then NextMove := ccNone;
             MoveResult := true;
             self.Paint;
           end;

           // log
             if Trace<>0 then SendMessage(Trace,WM_SetText,0,integer(@s[1]));
         end;
    end;

  end;

  end else inherited;
end;


// create an empty chessboard
constructor TChessBoard.CreateNew;
var
  i, j: integer;
begin
  inherited;
  fHistory := TStringList.Create;
  for i:=0 to 7 do begin
    for j:=0 to 7 do begin
      fField[i,j] := TChessCell.CreateNew;
    end;
  end;
  fBoard := nil;
end;

// remove a chessboard
Destructor TChessBoard.Destroy;
var
  i, j: integer;
begin
  fHistory.Free;
  for i:=0 to 7 do begin
    for j:=0 to 7 do begin
      fField[i,j].Free;
    end;
  end;
  if assigned(fBoard) then fBoard.Free;
  fBoard := nil;
  inherited;
end;

// fill the chessboard with all pieces in initial configuration
procedure TChessBoard.Initialize;
var
  i, j: integer;
begin
  fHistory.Clear;
  for i:=0 to 7 do fField[i,0].Color := ccWhite;
  fField[0,0].Piece := cpRook;
  fField[1,0].Piece := cpHorse;
  fField[2,0].Piece := cpBishop;
  fField[3,0].Piece := cpQueen;
  fField[4,0].Piece := cpKing;
  fField[5,0].Piece := cpBishop;
  fField[6,0].Piece := cpHorse;
  fField[7,0].Piece := cpRook;
  for i:=0 to 7 do begin
    fField[i,1].Color := ccWhite;
    fField[i,1].Piece := cpPawn;
  end;
  for i:=0 to 7 do begin
    for j:=2 to 5 do begin
      fField[i,j].Color := ccNone;
      fField[i,j].Piece := cpNOne;
    end;
  end;
  for i:=0 to 7 do begin
    fField[i,6].Color := ccBlack;
    fField[i,6].Piece := cpPawn;
  end;
  for i:=0 to 7 do fField[i,7].Color := ccBlack;
  fField[0,7].Piece := cpRook;
  fField[1,7].Piece := cpHorse;
  fField[2,7].Piece := cpBishop;
  fField[3,7].Piece := cpQueen;
  fField[4,7].Piece := cpKing;
  fField[5,7].Piece := cpBishop;
  fField[6,7].Piece := cpHorse;
  fField[7,7].Piece := cpRook;

end;

// export a configuration into a StringList
procedure TChessBoard.ExportToList(aList: TStringList);
var
  col, row: integer;
  s: string;
begin
  aList.Clear;
  for row:=7 downto 0 do begin
    s := '';
    for col:=0 to 7 do begin
      case fField[col,row].Piece of
        cpNone:   s := s + '*';
        cpPawn:   if fField[col,row].Color=ccWhite then s := s + 'P' else s := s + 'p';
        cpRook:   if fField[col,row].Color=ccWhite then s := s + 'R' else s := s + 'r';
        cpHorse:  if fField[col,row].Color=ccWhite then s := s + 'H' else s := s + 'h';
        cpBishop: if fField[col,row].Color=ccWhite then s := s + 'B' else s := s + 'b';
        cpQueen:  if fField[col,row].Color=ccWhite then s := s + 'Q' else s := s + 'q';
        cpKing:   if fField[col,row].Color=ccWhite then s := s + 'K' else s := s + 'k';
      end;
    end;
    aList.Add(s);
  end;
end;

// import a configuration from a StringList
procedure TChessBoard.ImportFromList(aList: TStringList);
var
  col, row, n: integer;
  s: string;
const
  pList = '*PRHBQKprhbqk';
begin
  for row:=7 downto 0 do begin
    s := aList.Strings[row];
    for col:=0 to 7 do begin
      n := pos(s[col+1],pList) - 1;
      case n of
        0:  begin   // *
              fField[col,7-row].Color := ccNone;
              fField[col,7-row].Piece := cpNone;
            end;
        1:  begin   // P
              fField[col,7-row].Color := ccWhite;
              fField[col,7-row].Piece := cpPawn;
            end;
        2:  begin   // R
              fField[col,7-row].Color := ccWhite;
              fField[col,7-row].Piece := cpRook;
            end;
        3:  begin   // H
              fField[col,7-row].Color := ccWhite;
              fField[col,7-row].Piece := cpHorse;
            end;
        4:  begin   // B
              fField[col,7-row].Color := ccWhite;
              fField[col,7-row].Piece := cpBishop;
            end;
        5:  begin   // Q
              fField[col,7-row].Color := ccWhite;
              fField[col,7-row].Piece := cpQueen;
            end;
        6:  begin   // K
              fField[col,7-row].Color := ccWhite;
              fField[col,7-row].Piece := cpKing;
            end;
        7:  begin   // p
              fField[col,7-row].Color := ccBlack;
              fField[col,7-row].Piece := cpPawn;
            end;
        8:  begin   // r
              fField[col,7-row].Color := ccBlack;
              fField[col,7-row].Piece := cpRook;
            end;
        9:  begin   // h
              fField[col,7-row].Color := ccBlack;
              fField[col,7-row].Piece := cpHorse;
            end;
        10: begin   // b
              fField[col,7-row].Color := ccBlack;
              fField[col,7-row].Piece := cpBishop;
            end;
        11: begin   // q
              fField[col,7-row].Color := ccBlack;
              fField[col,7-row].Piece := cpQueen;
            end;
        12: begin   // k
              fField[col,7-row].Color := ccBlack;
              fField[col,7-row].Piece := cpKing;
            end;
      end;
    end;
  end;
end;

// save current situation to history
procedure TChessBoard.PushContext;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  ExportToList(sl);
  fHistory.Add(sl.Text);
  sl.Free;
end;

// retreive current situation from history
procedure TChessBoard.PopContext;
var
  sl: TStringList;
begin
  sl := TStringList.Create;
  sl.Text := fHistory.Strings[fHistory.Count-1];
  fHistory.Delete(fHistory.Count-1);
  ImportFromList(sl);
  sl.Free;
end;

// draw the full chessboard with all pieces
procedure TChessBoard.DrawBoard;
begin
  Board.Paint;
end;

// copies the virtual situation from the ChessBord component into the ChessPanel component
procedure TChessBoard.CopySituation;
var
  row, col: integer;
begin
  if not assigned(Board) then exit;
  for row:=0 to 7 do begin
    for col:=0 to 7 do begin
      Board.fField[row,col] := fField[row,col];
    end;
  end;
end;


{
   ****************************************************************
   ****************************************************************
   ******************* Delphi / Panoramic wrapper *****************
   ****************************************************************
   ****************************************************************
}

// test function for devellopment purposes
function TestChess(hMemo: HWND): integer; stdcall; export;
var
  Field: TChessBoard;
  sl: TStringList;
  s: string;
begin
  sl := TStringList.Create;
  Field := TChessBoard.CreateNew;
  Field.Initialize;
  Field.ExportToList(sl);

  s := sl.Text;
  SendMessage(hmemo,WM_SETTEXT,0,integer(pchar(s)));
  result := 0;
  sl.Free;
end;
exports TestChess;

// create a complete chessboard component and display it into a destination component (may be a Form, a Panel etc)
// Panoramic version :
function CreateChessBoard(hDest: HWND; siz, x, y: integer; ColorDark, ColorLight: TColor): integer; stdcall; export;
var
  CB: TChessBoard;
  fic: string;

  procedure ExtractResToFile(ResType, ResName, ResNewName : String);
  var
    Res : TResourceStream;
  begin
    Res := TResourceStream.Create(Hinstance, Resname, Pchar(ResType));
    Res.SavetoFile(ResNewName);
    Res.Free;
  end;

begin
  result := 0;
  try
    if (not siz=16) and (not siz=48) then exit;
    if hDest=0 then exit;
    CB := TChessBoard.CreateNew;
    CB.Initialize;
    CB.Board := TChessPanel.Create(nil);
    CB.Board.ParentWindow := hDest;
    CB.Board.Width := 1 + 8*(siz+1) + 1;
    CB.Board.Height := CB.Board.Width;
    CB.Board.Top := y;
    CB.Board.Left := x;
    CB.Board.Dark := ColorDark;
    CB.Board.Light := ColorLight;
    CB.Board.IconSize := siz;
    CB.Board.BlackPawnDoubleAdvance := Point(-1,-1);
    CB.Board.WhitePawnDoubleAdvance := Point(-1,-1);
    CB.Board.WhiteKingMoved := false;
    CB.Board.LeftWhiteRookMoved := false;
    CB.Board.RightWhiteRookMoved := false;
    CB.Board.BlackKingMoved := false;
    CB.Board.LeftBlackRookMoved := false;
    CB.Board.RightBlackRookMoved := false;

    CB.Board.OnMouseDown := CB.Board.MyMouseDown;
    CB.Board.OnMouseUp := CB.Board.MyMouseUp;
    case siz of
      16: begin
            fic := 'KGF_temp.ilb';
            if not FileExists(fic) then ExtractResToFile('DATAFILE','TOOLBAR_ICONS','.\KGF_temp.ilb');  // rcuprer le fichier ilb
            CB.Board.ImageList := TImageList.Create(nil);       // crer une imagelist temporaire
            CB.Board.ImageList.DrawingStyle := dsTransparent;
            CB.Board.ImageList.BkColor := clBtnFace;
            CB.Board.ImageList.Height := siz;
            CB.Board.ImageList.Width := siz;
            CB.Board.IconSize := siz;
            ReadComponentResFile(fic, CB.Board.ImageList) ;   // charger le fichier ilb dans cette liste
            CB.Board.ImageOffset := 382;
          end;
      48: begin
            fic := 'Chess_temp.ilb';
            if not FileExists(fic) then ExtractResToFile('DATAFILE','CHESS_ICONS','.\Chess_temp.ilb');  // rcuprer le fichier ilb
            CB.Board.ImageList := TImageList.Create(nil);       // crer une imagelist temporaire
            CB.Board.ImageList.DrawingStyle := dsTransparent;
            CB.Board.ImageList.BkColor := clBtnFace;
            CB.Board.ImageList.Height := siz;
            CB.Board.ImageList.Width := siz;
            CB.Board.IconSize := siz;
            ReadComponentResFile(fic, CB.Board.ImageList) ;   // charger le fichier ilb dans cette liste
            CB.Board.ImageOffset := 382;
            CB.Board.ImageOffset := 0;
          end;
    end;
    CB.CopySituation;
    CB.Board.NextMove := ccWhite;
    CB.DrawBoard;
    result := integer(CB);
  except
  end;
end;
exports CreateChessBoard;

// create a complete chessboard component and display it into a destination component (may be a Form, a Panel etc)
//Delphi version (no 16 bit icons, only 48 bits !)
function CreateChessBoardForDelphi(hDest: HWND; siz, x, y: integer; ColorDark, ColorLight: TColor): TChessBoard; stdcall; export;
var
  CB: TChessBoard;
  fic: string;
begin
  result := nil;
  try
    if (not siz=16) and (not siz=48)  and (not siz=72) then exit;
    if hDest=0 then exit;
    CB := TChessBoard.CreateNew;
    CB.Initialize;
    CB.Board := TChessPanel.Create(nil);
    CB.Board.ParentWindow := hDest;
    CB.Board.Width := 1 + 8*(siz+1) + 1;
    CB.Board.Height := CB.Board.Width;
    CB.Board.Top := y;
    CB.Board.Left := x;
    CB.Board.Dark := ColorDark;
    CB.Board.Light := ColorLight;
    CB.Board.IconSize := siz;
    CB.Board.BlackPawnDoubleAdvance := Point(-1,-1);
    CB.Board.WhitePawnDoubleAdvance := Point(-1,-1);
    CB.Board.WhiteKingMoved := false;
    CB.Board.LeftWhiteRookMoved := false;
    CB.Board.RightWhiteRookMoved := false;
    CB.Board.BlackKingMoved := false;
    CB.Board.LeftBlackRookMoved := false;
    CB.Board.RightBlackRookMoved := false;

    CB.Board.OnMouseDown := CB.Board.MyMouseDown;
    CB.Board.OnMouseUp := CB.Board.MyMouseUp;
            // the library  'Chess_48.ilb' contains the 12 chess icons
            fic := 'Chess_48.ilb'; 
            CB.Board.ImageList := TImageList.Create(nil);       // crer une imagelist temporaire
            CB.Board.ImageList.DrawingStyle := dsTransparent;
            CB.Board.ImageList.BkColor := clBtnFace;
            CB.Board.ImageList.Height := siz;
            CB.Board.ImageList.Width := siz;
            CB.Board.IconSize := siz;
            ReadComponentResFile(fic, CB.Board.ImageList) ;   // charger le fichier ilb dans cette liste
            CB.Board.ImageOffset := 382;
            CB.Board.ImageOffset := 0;
    CB.CopySituation;
    CB.Board.NextMove := ccWhite;
    CB.DrawBoard;
    result := CB;
  except
  end;
end;
exports CreateChessBoardForDelphi;


function SetChessBoardTrace(CB: TChessBoard; trace: HWND): integer; stdcall; export;
begin
  result := -1;
  try
    if not assigned(CB) then exit;
    if CB.ClassName<>'TChessBoard' then exit;
    CB.Board.Trace := trace;
    result := 0;
  except
  end;
end;
exports SetChessBoardTrace;

function MoveChessBoardPiece(CB: TChessBoard; StartCol, StartRow, EndCol, EndRow: integer; Roque: TChessRoque): integer; stdcall; export;
var
  xo, yo, xd, yd: integer;
  s: string;
begin
  result := -1;                                                                 // erreur technique
  try
    if not assigned(CB) then exit;
    if CB.ClassName<>'TChessBoard' then exit;
    case Roque of
      crNone:  begin         // pas de roque - mouvement normal de position  position
                 if (StartRow<1) or (StartRow>8) then exit;
                 if (StartCol<1) or (StartCol>8) then exit;
                 if (EndRow<1) or (EndRow>8) then exit;
                 if (EndCol<1) or (EndCol>8) then exit;
                 xo := 1 + (StartCol-1)*(CB.Board.IconSize+1) + trunc(CB.Board.IconSize/2);
                 yo := 1 + (StartRow-1)*(CB.Board.IconSize+1) + trunc(CB.Board.IconSize/2);
                 xd := 1 + (EndCol-1)*(CB.Board.IconSize+1) + trunc(CB.Board.IconSize/2);
                 yd := 1 + (EndRow-1)*(CB.Board.IconSize+1) + trunc(CB.Board.IconSize/2);
                 CB.Board.MyMouseDown(nil,mbLeft,[],xo,yo);
                 CB.Board.MyMouseUp(nil,mbLeft,[],xd,yd);
                 if CB.Board.MoveResult then result := 0;                       // dplacement effectu avec succs
               end;
      crSmall: begin         // petit roque du joueur dont c'est le tour
                 if IsRoqueLegal(CB.Board,Roque) then begin
                   if CB.Board.NextMove=ccWhite then begin
                     CB.Board.Move(4,0,6,0);
                     CB.Board.Move(7,0,5,0);
                     CB.Board.MoveResult := true;
                     CB.Board.NextMove := ccBlack;
                     CB.Board.Paint;

                     // log
                     s := 'Blanc - petit roque';
                     if CB.Board.Trace<>0 then SendMessage(CB.Board.Trace,WM_SETTEXT,0,integer(@s[1]));
                   end else begin
                     CB.Board.Move(4,7,6,7);
                     CB.Board.Move(7,7,5,7);
                     CB.Board.MoveResult := true;
                     CB.Board.NextMove := ccWhite;
                     CB.Board.Paint;

                     // log
                     s := 'Noir - petit roque';
                     if CB.Board.Trace<>0 then SendMessage(CB.Board.Trace,WM_SETTEXT,0,integer(@s[1]));
                   end;
                 end;
               end;
      crLarge: begin         // grand roque du joueur dont c'est le tour
                 if IsRoqueLegal(CB.Board,Roque) then begin
                   if CB.Board.NextMove=ccWhite then begin
                     CB.Board.Move(4,0,2,0);
                     CB.Board.Move(0,0,3,0);
                     CB.Board.MoveResult := true;
                     CB.Board.NextMove := ccBlack;
                     CB.Board.Paint;

                     // log
                     s := 'Blanc - grand roque';
                     if CB.Board.Trace<>0 then SendMessage(CB.Board.Trace,WM_SETTEXT,0,integer(@s[1]));
                   end else begin
                     CB.Board.Move(4,7,2,7);
                     CB.Board.Move(0,7,3,7);
                     CB.Board.MoveResult := true;
                     CB.Board.NextMove := ccWhite;
                     CB.Board.Paint;

                     // log
                     s := 'Noir - grand roque';
                     if CB.Board.Trace<>0 then SendMessage(CB.Board.Trace,WM_SETTEXT,0,integer(@s[1]));
                   end;
                 end;
               end;
    end;
    result := 0;
  except
  end;
end;
exports MoveChessBoardPiece;

function GetStringFromMemo(hMemo: HWND): string;
var
  s1: string;
  Len: integer;

    function GetClassName(Handle: THandle): String;
    var
      Buffer: array[0..MAX_PATH] of Char;
    begin
      Windows.GetClassName(Handle, @Buffer, MAX_PATH);
      Result := String(Buffer);
    end;

    procedure Delay(AMSecs: Cardinal);
    var
      iStop : cardinal;
    begin
      iStop := GetTickCount + AMsecs;
      while GetTickCount < iStop do
      begin
        Forms.Application.ProcessMessages;
        sleep(1);
      end
    end;

begin
    try
      if GetClassName(hMemo)='TMemo' then begin

        Len := SendMessage(hMemo, WM_GETTEXTLENGTH, 0, 0);
        if Len>0 then begin
          SetLength(s1, Len+1);
          Len := SendMessage(hMemo, WM_GETTEXT, Len+1, LPARAM(PChar(s1)));
          delay(100);
          SetLength(s1, Len);
        end;

        result := Trim(s1);
        exit;
      end;
    except
    end;
end;

function PopulateChessBoard(CB: TChessBoard; aSituation: integer; StartColor: TChessColor): integer; stdcall; export;
var
  s: string;
  sl: TStringList;
  i, j, row, col, ind: integer;
const
  pList = '*PRHBQKprhbqk';
begin
  result := -1;                                                                 // erreur technique
  try
    if not assigned(CB) then exit;
    if CB.ClassName<>'TChessBoard' then exit;
    if aSituation=0 then exit;
    if (StartColor<>ccWhite) and (StartColor<>ccBlack) then exit;
    s := GetStringFromMemo(aSituation);
    if RightStr(s,2)<>#13#10 then s := s + #13#10;
    if length(s)<>80 then exit;
    sl := TStringList.Create;
    sl.Text := s;
    if sl.Count=8 then begin
      for i:=0 to sl.Count-1 do begin
        s := sl.Strings[i];
        if length(s)<>8 then begin
          sl.Free;
          exit;
        end;
        for j:=1 to 8 do  begin
          if pos(s[j],pList)=0 then begin
            sl.Free;
            exit;
          end;
        end;
      end;
      for row:=0 to 7 do begin
        s := sl.Strings[7-row];
        for col:=0 to 7 do begin
          ind := pos(s[col+1],pList);
          case ind of
            1: begin
                 CB.Board.Piece[col,row] := cpNone;
                 CB.Board.Color[col,row] := ccNone;
               end;
            2: begin
                 CB.Board.Piece[col,row] := cpPawn;
                 CB.Board.Color[col,row] := ccWhite;
               end;
            3: begin
                 CB.Board.Piece[col,row] := cpRook;
                 CB.Board.Color[col,row] := ccWhite;
               end;
            4: begin
                 CB.Board.Piece[col,row] := cpHorse;
                 CB.Board.Color[col,row] := ccWhite;
               end;
            5: begin
                 CB.Board.Piece[col,row] := cpBishop;
                 CB.Board.Color[col,row] := ccWhite;
               end;
            6: begin
                 CB.Board.Piece[col,row] := cpQueen;
                 CB.Board.Color[col,row] := ccWhite;
               end;
            7: begin
                 CB.Board.Piece[col,row] := cpKing;
                 CB.Board.Color[col,row] := ccWhite;
               end;

            8: begin
                 CB.Board.Piece[col,row] := cpPawn;
                 CB.Board.Color[col,row] := ccBlack;
               end;
            9: begin
                 CB.Board.Piece[col,row] := cpRook;
                 CB.Board.Color[col,row] := ccBlack;
               end;
            10: begin
                 CB.Board.Piece[col,row] := cpHorse;
                 CB.Board.Color[col,row] := ccBlack;
               end;
            11: begin
                 CB.Board.Piece[col,row] := cpBishop;
                 CB.Board.Color[col,row] := ccBlack;
               end;
            12: begin
                 CB.Board.Piece[col,row] := cpQueen;
                 CB.Board.Color[col,row] := ccBlack;
               end;
            13: begin
                 CB.Board.Piece[col,row] := cpKing;
                 CB.Board.Color[col,row] := ccBlack;
               end;
          end;
        end;
      end;
      CB.Board.NextMove := StartColor;
      CB.Board.WhiteKingMoved := false;
      CB.Board.BlackKingMoved := false;
      CB.Board.LeftWhiteRookMoved := false;
      CB.Board.LeftBlackRookMoved := false;
      CB.Board.RightWhiteRookMoved := false;
      CB.Board.RightBlackRookMoved := false;
      CB.Board.Paint;
      result := 0;
    end;
    sl.Free;
  except
  end;
end;
exports PopulateChessBoard;

function DeleteChessBoard(CB: TChessBoard): TChessBoard; stdcall; export;
begin
  result := CB;                                                                 // erreur technique
  try
    if not assigned(CB) then exit;
    if CB.ClassName<>'TChessBoard' then exit;
    CB.Free;
    result := nil;
  except
  end;
end;
exports DeleteChessBoard;

function ResizeChessBoard(CB: TChessBoard; siz: integer): integer; stdcall; export;
var
  w, h: integer;
begin
  result := -1;                                                                 // erreur technique
  try
    if not assigned(CB) then exit;
    if CB.ClassName<>'TChessBoard' then exit;
    if (siz<>48) and (siz<>72) then exit;

    w := 1 + 8*(siz+1);
    h := w;
    CB.Board.IconSize := siz;
    CB.Board.Width := w;
    CB.Board.Height := w;
    CB.Board.Paint;
    result := 0;
  except
  end;
end;
exports ResizeChessBoard;

end.
